from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject
from OFS.SimpleItem import SimpleItem
from DateTime import DateTime
from Globals import HTMLFile, InitializeClass, package_home
from AccessControl import ClassSecurityInfo

from Permissions import p_manageAggregationTool, p_manageRSSCollection

from Products.Xron.XronDTMLMethod import XronDTMLMethod

from RSSFeed import RSSFeed, default_fields

#Supports Syndic8 services
from Syndic8 import Syndic8Server

#For debugging
from logger import logger

#For skin definitions
import os.path

#Aggregation Tool
class AggregationTool (UniqueObject, SimpleItem, ActionProviderBase):
    """ 
        The aggregation tool manages the site-wide policies for
        aggregation. It provides new type in CMF called RSSFeed,
        users can import RSSFeeds from Syndic8, Meerkat (Not implement in
        this version) then select what RSSFeeds will be parsed and 
        aggregated it's news headlines.
    """

    id = 'portal_aggregation'

    meta_type= 'CMF Aggregation Tool'

    security = ClassSecurityInfo()

    #ZMI Methods
    manage_options = ( ActionProviderBase.manage_options
                     + ( { 'label'  : 'Overview'
                         , 'action' : 'manage_overview'
                         , 'help'   : ( 'CMFDefault'
                                      , 'Syndication-Tool_Overview.stx' )
                         }
                        ,{ 'label'  : 'Properties'
                         , 'action' : 'manage_properties'
                         , 'help'   : ( 'CMFDefault'
                                      , 'Syndication-Tool_Overview.stx' )
                         }
                        ,{ 'label'  : 'Policies'
                         , 'action' : 'manage_policies'
                         , 'help'   : ( 'CMFDefault'
                                      , 'Syndication-Tool_Overview.stx' )
                         }
                        ,{ 'label'  : 'Reports'
                         , 'action' : 'manage_reports'
                         , 'help'   : ( 'CMFDefault'
                                      , 'Syndication-Tool_Overview.stx' )
                         }
                        )
                     )

    _dtmldir = os.path.join( package_home( globals() ), 'dtml' )
    security.declareProtected(p_manageAggregationTool, 'manage_overview')
    manage_overview = HTMLFile('manage_overview', _dtmldir)

    security.declareProtected(p_manageAggregationTool, \
                              'manage_properties')
    manage_properties = HTMLFile('manage_properties', _dtmldir )

    security.declareProtected(p_manageAggregationTool, 'Policies')
    Policies = HTMLFile('manage_policies', _dtmldir)

    security.declareProtected(p_manageAggregationTool, 'manage_reports')
    manage_reports = HTMLFile('manage_reports', _dtmldir)

    #Default Sitewide Values
    isAllowed = 0

    max_feeds = 50

    SchemeMap = {'DMOZ':['Games', 'Computers'], 'HV':['Science', 'Bussiness']}

    allowedSchemeMap = SchemeMap.copy()

    #OK
    def __init__(self, id='portal_aggregation', title='Manage site wide services for aggregation'):
        """ """
        self.id = id
        self.title = title

    #OK
    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
            Return actions provided by tool
        """
        return self._actions

    security.declarePublic('buildFrequencyMap')
    def frequencyValues(self):
        """
            Return a list of possible refresh Frequency.
        """
        return ['minutes','hours', 'days', 'weeks', 'months', 'years']

    #OK - 4/26/2003
    security.declareProtected(p_manageAggregationTool, 'editProperties')
    def editProperties( self
                      , max_feeds=None
                      , isAllowed=None
                      , Schemes = []
                      , REQUEST=None
                      ):
        """
            Edit the properties for the AggregationTool.
        """

        if isAllowed is not None:
            self.isAllowed = isAllowed

        if Schemes:
            self.allowedSchemeMap = {}
            for scheme in Schemes:
                self.allowedSchemeMap[scheme] = getattr(REQUEST, scheme + '_Categories', [])

        if max_feeds:
            self.max_feeds = max_feeds

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect( self.absolute_url()
                                        + '/manage_properties'
                                        + '?manage_tabs_message=Tool+Updated.'
                                     )

    #For Scheduling #######################################################################
    security.declareProtected(p_manageRSSCollection , 'setSchedule')
    def setSchedule (self, obj, refreshBase=None, refreshPeriod=None, refreshFrequency=None):
        """ """
        if self.hasSchedule(obj):
            self.delSchedule(obj)

        id = 'collection_schedule'
        executeAt = refreshBase or DateTime()
        periodDays = refreshPeriod or 0
        frequency = refreshFrequency or 'hours'

        XronTrigger = XronDTMLMethod(id = id, 
                                     executeAt = executeAt,
                                     periodDays = periodDays,
                                     frequency = frequency,
                                     title='')

        dtml_codes = "<dtml-call proxy_call>"
        XronTrigger.munge(dtml_codes)

        #Set proxy roles for this
        #XronTrigger.manage_proxy('Manager')
        obj._setObject(id, XronTrigger)

    security.declareProtected(p_manageRSSCollection , 'delSchedule')
    def delSchedule (self, obj):
        """ """
        obj._delObject('collection_schedule')

    security.declarePublic('hasSchedule')
    def hasSchedule (self, obj):
        """ """
        if hasattr(obj.aq_explicit, 'collection_schedule'):
            return 1
        else:
            return 0

    #4/26/2003
    security.declarePublic('getRefreshPeriod')
    def getRefreshPeriod( self, obj):
        """
            Return the refresh period for .
        """

        if not self.isSiteAggregationAllowed(): 
            raise 'Aggregation is Not Allowed'

        col_schedule = getattr(obj.aq_explicit, 'collection_schedule')
        if col_schedule:
            return col_schedule.periodDays
        else:
            raise 'No schedule found'
    
    #4/26/2003
    security.declarePublic('getRefreshFrequency')
    def getRefreshFrequency(self, obj):
        """
        """
        if not self.isSiteAggregationAllowed():
            raise 'Aggregation is not Allowed'

        col_schedule = getattr(obj.aq_explicit, 'collection_schedule')
        if col_schedule:
            return col_schedule.frequency
        else:
            raise 'No schedule found'

    #4/26/2003
    security.declarePublic('getRefreshBase')
    def getRefreshBase(self, obj):
        """  
        """
        if not self.isSiteAggregationAllowed(): 
            raise 'Aggregation is not Allowed'

        col_schedule = getattr(obj.aq_explicit, 'collection_schedule')
        if col_schedule:
            return col_schedule.executeAt.ISO()
        else:
            raise 'No schedule found'
    ##########################################################################################

    #OK
    security.declarePublic('isSiteAggregationAllowed')
    def isSiteAggregationAllowed(self):
        """
            Return site-wide aggregation policy
        """
        return self.isAllowed

    #4/26/2003
    #Need to change Interface name -> resetSchemeMap or something ...
    security.declareProtected(p_manageAggregationTool, 'refreshSchemeMap')
    def refreshSchemeMap(self, Schemes=[], REQUEST=None):
        """
           ReImport available schemes from Syndic8 Server into portal
        """
        Server = Syndic8Server()
        new_dict = {}
        if not Schemes:
            Schemes = Server.getCategorySchemes()

        for Scheme in Schemes:
            new_dict[Scheme] = Server.getCategoryTree(Scheme)
        self.SchemeMap = new_dict

        allowedSchemeMap = self.SchemeMap.copy()

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect( self.absolute_url()
                                     + '/manage_properties'
                                     + '?manage_tabs_message=Schemes: %s +Refreshed.' %`Schemes`
                                      )

    #OK
    security.declarePublic('getSchemes')
    def getSchemes(self):
        """
            Return the all available schemes from Syndic8 server
        """
        if not self.isSiteAggregationAllowed(): 
            raise 'Aggregation is not Allowed'

        #if not self.SchemeMap:
        #    self.refreshSchemeMap()

        return self.SchemeMap.keys()

    #4/26/2003
    security.declarePublic('getCategories')
    def getCategories(self, scheme):
        """
            Return the full categories for specific scheme
        """
        if not self.isSiteAggregationAllowed(): 
            raise 'Aggregation is not Allowed'

        #if not self.SchemeMap:
        #    self.refreshSchemeMap()

        try: 
            return self.SchemeMap[scheme]
        except KeyError:
            return [] #No scheme no returned categories

    #OK
    security.declarePublic('getAllowedSchemes')
    def getAllowedSchemes(self):
        """
            Return the list of site-wide allowed schemes
        """
        if not self.isSiteAggregationAllowed(): 
            raise 'Aggregation is not Allowed'

        return self.allowedSchemeMap.keys()

    #OK
    security.declarePublic('getAllowedCategories')
    def getAllowedCategories(self, scheme):
        """
            Return the allowed categories for specific scheme
        """
        if not self.isSiteAggregationAllowed(): 
            raise 'Aggregation is not Allowed'

        try: 
            return self.allowedSchemeMap[scheme]
        except KeyError:
            return [] #unallowable scheme so unallowable it's categories
    
    #4/26/2003
    security.declarePublic('getMaxFeeds')
    def getMaxFeeds(self):
        """
            Return the max_feeds to be aggregated in the RSSCollection
        """

        if not self.isSiteAggregationAllowed(): 
            raise 'Aggregation is not Allowed'

        return self.max_feeds

    #4/26/2003
    security.declarePublic('getDefaultFields')
    def getDefaultFields(self):
        """ """
        return default_fields

    #OK
    security.declarePublic('getFeeds')
    def getFeeds(self, ids, fields=[]):
        """ """

        Server = Syndic8Server()

        if not fields:
            fields = self.getDefaultFields()

        return Server.getFeeds(ids, fields)

    security.declarePublic('search')
    def search(self, pattern, scheme_map={}, fields=[]):
        """
            syndic8.FindFeeds(string Pattern to match against all feed field)  -> feedids
            
            syndic8.FindSites(string Pattern to match against SiteURL field) - > feedids
            
            syndic8.QueryFeeds(string Feed field to match, string Relational operator, string Value to match)
            ->array of FeedIDs of matching feeds
            This function takes the given feed field, relationship (<, >, <=. >=, !=, =, like, or regexp), 
            and feed value and returns the list of feeds with a matching value in the field.
        """

        Server = Syndic8Server()

        if not fields:
            fields = self.getDefaultFields()

        results = Server.search(pattern=pattern, scheme_map=scheme_map, fields = fields)

        return results

InitializeClass(AggregationTool)