from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Globals import InitializeClass
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from binascii import b2a_base64
from md5 import md5
from rssparser import parse
from RSSStory import RSSStory
from RSSFeed import RSSFeed, STATUS_DEAD, STATUS_OK
from Permissions import p_manageRSSCollection, p_View

#For scheduling
from DateTime import DateTime

#For debugging
from logger import logger

import string
_replace_table = [
    ('_', '/:%$ \t'),
    ('-', '=+!'),
    (',', '.&@?*<>#;'),
    ]

_delete_chars = '\'"'
_replace_chars = ''.join([ x[1] for x in _replace_table ])
_replace_by = ''.join([ (x[0] * len(x[1])) for x in _replace_table ])
_base64_to_id_trans = string.maketrans( _replace_chars, _replace_by )

factory_type_information = ({'id': 'RSSCollection',
                             'content_icon': 'folder_icon.gif',
                             'meta_type': 'RSSCollection',
                             'description':""" """,
                             'product': 'CMFAggregator',
                             'factory': 'addRSSCollection',
                             'immediate_view': 'rsscollection_edit_form',
                             'allowed_content_types':['RSSFeed', 'RSSStory', 'RSSCollection'],
                             'actions':({'id': 'view',
                                          'name': 'View',
                                          'action': 'rsscollection_view',
                                          'permissions': (p_View,)},
                                         {'id': 'contents',
                                          'name': 'Contents',
                                          'action': 'folder_contents',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'edit',
                                          'name': 'Edit',
                                          'action': 'portal_form/rsscollection_edit_form',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'import',
                                          'name': 'Import',
                                          'action': 'rsscollection_import_form',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'schedule',
                                          'name': 'Schedule',
                                          'action': 'rsscollection_schedule_form',
                                          'permissions': (p_manageRSSCollection,)},
                                        ),
                             },
                            )

def addRSSCollection( self, id, title='', description=''):
    """ """
    o=RSSCollection( id=id, 
                     title=title, 
                     description=description
                    )

    self._setObject(id,o)

class RSSCollection(SkinnedFolder):
    """ """
    meta_type = 'RSSCollection'

    security = ClassSecurityInfo()

    def __init__(self, id, title='', description=''):
        """ """
        self.id = id
        self.title = title
        self.description = description

    def edit(self,id='', title='', description=''):
        """ """
        if id:
            self.id = id
        self.title = title
        self.description = description

    security.declarePrivate('syncStory')
    def syncStory(self,id, fields):
        """ """
        pass

    security.declarePrivate('newStory')
    def newStory(self, id, fields):
        """ """
        (title, description, link) = fields
        self.invokeFactory(type_name='RSSStory',
                           id=id, 
                           title=title, 
                           description=description,
                           link=link
                           )
        new_story = getattr(self.aq_explicit, id)

        #This will raise an error when calling from Xron schedule - Why???
        #workflows = self.portal_workflow.doActionFor(new_story, 
        #                                             'publish',
        #                                              comment='Automatically published when aggregating',
        #                                             )
        #~~

    security.declarePrivate('getStoryId')
    def getStoryId(self, fields):
        """ """
        id = b2a_base64(md5(''.join(fields)).digest()).strip()
        id = id.translate(_base64_to_id_trans, _delete_chars)

        while id and id[0] == '_':
            id=id[1:]

        if not id:
            id = 'error_id'

        return id

    security.declarePrivate('parseStory')
    def parseStory(self, story):
        """ """
        fields = story.get('title', 'unknown'), story.get('description',''), story.get('link','')

        id = self.getStoryId(fields=fields)

        if hasattr(self.aq_explicit, id):
            self.syncStory(id, fields=fields)
        else:
            self.newStory(id, fields=fields)

    security.declarePrivate('parseFeed')
    def parseFeed(self, feedobj):
        """ parse feed object to get stories """

        dataurl = feedobj.dataurl

        try:
            logger(0, 'Try to parse url: %s' %dataurl)
            result = parse(dataurl)
            logger(0, "Cheer, it's OK, results: %s" % `result`)
        except:
            logger(0, 'Fail')
            result = {'items':[]}

        for item in result['items']:
            self.parseStory(item)

    security.declareProtected(p_manageRSSCollection , 'aggregate')
    def aggregate(self):
        """ Now, let it runs with the feeds """
        ########################################################
        #Run this method the context of this user - Very dangerous
        #acl_users = self.acl_users
        #authenticated_user = acl_users.getUser('hungbv')
        #authenticated_user = authenticated_user.__of__(acl_users)
        #self.app.REQUEST['AUTHENTICATED_USER'] = authenticated_user

        #from AccessControl import getSecurityManager
        #sm = getSecurityManager()
        #sm_context = sm._context
        #sm._context.user = authenticated_user
        #########################################################

        feeds = self.objectValues('RSSFeed')

        for feed in feeds:
            if feed.status==STATUS_DEAD: 
                logger(0, 'Bypass %s because it was dead!' %feed.getId())
                continue #bypass dead feed if u do not want, please reset it's status

            self.parseFeed(feed)

        logger(0, 'Aggregation Complete')


    security.declareProtected(p_manageRSSCollection , 'importFeeds')
    def importFeeds(self, feeds):
        """ 
        """
        for feed in feeds:
            title = '%(sitename)s (%(siteurl)s, %(dataurl)s)' %feed
            dataurl = feed['dataurl']
            description = feed['description']
            fetchable = feed['fetchable']
            status = feed['status']
            categories = feed['Categories']

            id = b2a_base64(md5(dataurl).digest()).strip()
            id = id.translate(_base64_to_id_trans, _delete_chars)

            while id and id[0] == '_':
                id=id[1:]

            if not hasattr(self.aq_explicit, id):
                self.invokeFactory( type_name='RSSFeed', 
                                    id =id, 
                                    title=title, 
                                    description=description, 
                                    dataurl=dataurl, 
                                    fetchable=fetchable, 
                                    status=status, 
                                    categories=categories)

InitializeClass(RSSCollection)
