from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFDefault.Link import Link
import urlparse

from Permissions import p_manageRSSFeed, p_View

factory_type_information = ( { 'id'             : 'RSSFeed'
                             , 'meta_type'      : 'RSSFeed'
                             , 'description'    : """ """
                             , 'icon'           : 'link_icon.gif'
                             , 'product'        : 'CMFAggregator'
                             , 'factory'        : 'addRSSFeed'
                             , 'immediate_view' : 'metadata_edit_form'
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'rssfeed_view'
                                  , 'permissions'   : (p_View, )
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'portal_form/rssfeed_edit_form'
                                  , 'permissions'   : (p_manageRSSFeed, )
                                  }
                                , { 'id'            : 'metadata'
                                  , 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : (p_manageRSSFeed, )
                                  }
                                )
                             }
                           ,
                           )

#Available fields from Syndic8 
all_fields = ['feedid',
              'sitename',
              'siteurl',
              'dataurl',
              'imageurl',
              'webmaster',
              'editor',
              'publisher',
              'creator',
              'date_created',
              'date_approved',
              'date_xml_changed',
              's8_owner_userid',
              'scraped',
              'meta_scraped',
              'fetchable',
              'description',
              'origin',
              'lang_code',
              'status',
              'poll_status',
              'rss_version',
              'views',            
              'headlines_per_day',
              'headlines_rank',
              'toolkit_id',
              'toolkit_version',
              'cur_polling_interval',
              'last_poll_time',
              'last_pollid',
              'archivable',
              'ref_feedid',
              'amazon_keywords',
              'amazon_mode',
              'date_good_parse',
              'last_etag',
              'Locations',
              'Categories']

#Use these fields as a default for RSSFeed objects
default_fields = ['feedid',
                  'sitename',
                  'siteurl', 
                  'dataurl',
                  'description',
                  'fetchable',
                  'status',
                  'Categories']

statuses = ['Available','Dead']
STATUS_OK = statuses[0]
STATUS_DEAD = statuses[1]

def addRSSFeed( self
              , id
              , title=''
              , description=''
              , dataurl=''
              , fetchable= 1
              , status=STATUS_OK 
              , categories = []
              ):
    """
        Add a RSSFeed instance to 'self'.
    """
    o=RSSFeed( id, title, description, dataurl, fetchable, status, categories)
    self._setObject(id,o)

###################################################
#Provide new Portal Type called RSSFeed 
class RSSFeed(Link):
    """
        A RSS Feed
    """

    __implements__ = ( Link.__implements__, )

    meta_type = 'RSSFeed'

    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title=''
                , description=''
                , dataurl=''
                , fetchable = 1
                , status=STATUS_OK
                , categories = []
                ):
        """ New RSSFeed created """

        self.id=id
        self.title=title
        self.description=description
        self._editDataURL(dataurl)
        self.fetchable = 1
        self.status = status
        self.categories = categories
        self.format = 'text/url'

    security.declareProtected(p_manageRSSFeed, 'edit' )
    def edit(self, title='', description='', dataurl='', fetchable=1, status=STATUS_OK, categories=[]):
        """ Update and reindex. """
        self.title=title
        self.description=description
        self._editDataURL(dataurl)
        self.fetchable = 1
        self.status = status
        self.categories = categories

    security.declarePrivate( '_editDataURL' )
    def _editDataURL( self, dataurl ):
        """
            Edit the dataurl of the RSSFeed
        """
        tokens = urlparse.urlparse( dataurl, 'http' )
        if tokens[0] == 'http':
            if tokens[1]:
                # We have a nethost. All is well.
                url = urlparse.urlunparse(tokens)
            elif tokens[2:] == ('', '', '', ''):
                # Empty URL
                url = ''
            else:
                # Relative URL, keep it that way, without http:
                tokens = ('', '') + tokens[2:]
                url = urlparse.urlunparse(tokens)
        else:
            # Other scheme, keep original
            url = urlparse.urlunparse(tokens)
        self.dataurl = url

    security.declarePublic('SearchableText')
    def SearchableText(self):
        """
            All text for indexing including dataurl
        """
        return "%s %s %s" % (self.title, self.description, self.dataurl)

    # Public interfaces - May be in AggregationTool, not here ###
    security.declarePublic('getAvailableStatuses')
    def getAvailableStatuses(self):
        """
        """
        return statuses

InitializeClass( RSSFeed )