import xmlrpclib
import xmlrpcpatch

#For debugging
from logger import logger

#########################################################
#Class for supporting Syndic8 services
class Syndic8Server:
    """
       This class exposes many services from Syndic8 server 
       to AggregationTool.
    """

    #Initialize XML_RPC server
    def __init__(self):
        try:
            logger(0, 'Initialize Syndic8 Server')
            self.Server = xmlrpclib.ServerProxy('http://www.syndic8.com/xmlrpc.php').syndic8
        except:
            logger(0, 'Fail to initialize Syndic8 Server')

    def getCategorySchemes(self):
        """ Get all available schemes from Syndic8  """
        try:
            logger(0, 'Try to get schemes from Syndic8')
            schemes = self.Server.GetCategorySchemes()
            logger(0, 'Successful!')
            return schemes
        except:
            logger(0, 'Fail to get schemes from Syndic8')
            return []

    def getCategoryTree(self, scheme):
        """ Get all categories in a scheme """
        try:
            logger(0, 'Try to get category tree for scheme %s' %scheme)
            categories = self.Server.GetCategoryTree(scheme)
            logger(0, 'Successful!')
            return categories
        except:
            logger(0, 'Fail to get category tree for scheme %s' %scheme)
            return []

    def getCategoryRoots(self, scheme):
        """ Get all root categories in a scheme """
        try:
            logger(0, 'Try to get roots for scheme %s' %scheme)
            categories = self.Server.GetCategoryRoots(scheme)
            logger(0, 'Successful!')
            return categories
        except:
            logger(0, 'Fail to get roots for scheme %s' %scheme)
            return []

    def getFeedsInCategory(self, scheme, category):
        """ Get rss feed ids from (scheme, category) """
        try:
            logger(0, 'Try to get feeds for scheme %s, category %s' %(scheme, category))
            feedids = self.Server.GetFeedsInCategory(scheme, category)
            logger(0, 'Successful!')
            return feedids
        except:
            logger(0, 'Fail to get feeds for scheme %s, category %s' %(scheme, category))
            return []

    def dummyFeeds(self):
        """ """
        dummy = []
        for i in range(5):
            feed = {}
            feed['feedid'] = 'feedid%d' %i
            feed['siteurl'] = 'site url %d' %i
            feed['sitename'] = 'site name %d' %i
            feed['dataurl'] = 'feed dataurl %d' %i
            feed['description'] = 'feed description %d' %i
            feed['fetchable'] = 'feed fetchable %d' %i 
            feed['status'] = 'feed status %d' %i 
            feed['Categories'] = 'feed Categories %d' %i

            dummy.append(feed)
        return dummy

    def findFeeds(self, pattern):
        """ """
        try:
            logger(0,'Start to search with pattern: %s' %pattern)
            ids = self.Server.FindFeeds(pattern)
            logger(0, 'Results: %s' %str(ids))
            return ids
        except:
            logger(0, 'Fail')
            return []

    def search(self, pattern, scheme_map, fields):
        """ """
        schemes_results=[]
        pattern_results=[]
        last_results=[]

        if scheme_map:
            temp = {}
            for scheme in scheme_map.keys():
                categories = scheme_map[scheme]
                for category in categories:
                    ids = self.getFeedsInCategory(scheme, category)
                    for id in ids:
                        temp[id] = 'Appended' 
            if not temp: 
                return []
            else:
                schemes_results = temp.keys()

        if pattern:
            pattern_results = self.findFeeds(pattern)
            if not pattern_results: 
                return []

        if scheme_map and pattern:
            last_results = [x for x in schemes_results if x in pattern_results]
        elif scheme_map:
            last_results = schemes_results
        elif pattern:
            last_results = pattern_results

        feeds = self.getFeeds(last_results, fields)
        return feeds

    def getFeeds(self, ids, fields):
        """ """
        try:
            logger(0, 'Try to get feeds for ids %s' %str(ids))
            feeds = self.Server.GetFeedInfo(ids, fields)
            logger(0, 'Successful!')
            return feeds
        except:
            logger(300, 'Fail')
